use crate::{
    decrypt_file_buffered,
    errors::{CvfsErrors, CvfsResult},
    metadata::{FileTag, Metadata},
    open_rwlocked,
};
use multi_key_manager::{file::File, LiveKeyMaterial, OpeningNonce};
use ring::aead::OpeningKey;
use std::{
    io::{Read, Write},
    path::Path,
    sync::{Mutex, Weak},
};

/// an open active file
pub struct ActiveFile {
    pub(crate) id: FileTag,
    data: Vec<u8>,
    real_file: File,
}
impl ActiveFile {
    pub fn open_rwlocked(
        real_path: &Path,
        id: FileTag,
        file_key: &mut OpeningKey<OpeningNonce>,
    ) -> CvfsResult<Self> {
        let mut real_file = open_rwlocked(real_path)?;

        let data = if real_file.empty()? {
            Vec::new()
        } else {
            decrypt_file_buffered(&mut real_file, file_key)
                .map_err(|_generic_ring_error| CvfsErrors::LeafDecryptionError)?
        };

        Ok(Self {
            id,
            data,
            real_file,
        })
    }
}

/// These Handlers should reduce the time the main Mutex is held so concurrent access is possible
pub struct SecureFileHandler {
    pub(crate) metadata: Weak<Mutex<Metadata>>,
    pub(crate) id: FileTag,
    /// the current seek position
    pub(crate) offset: usize,
}
